@@ -138,8 +138,17 @@ def smooth_reference(reference, kernel_size=15):
     return reference_smooth
 
 
+def apply_polynomial(data, coefficients):
+    degree = len(coefficients)
+    out = np.zeros_like(data)
+    for i in range(degree):
+        out = out + coefficients[i] * np.power(data, i + 1)
+    return out
+
+
 def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_factor=1, do_truncation_correction=True,
-                             extension_fraction=0.1, dose_reduction=1., datatype=np.float32, keep_rows=None):
+                             extension_fraction=0.1, dose_reduction=1., datatype=np.float32, keep_rows=None,
+                             do_beam_hardening_correction=False, poly=None):
     '''Preprocess the raw images, i.e. remove shifts, transform into line integral domain.
     Projections are extended for truncation correction if needed.
 
@@ -152,6 +161,8 @@ def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_fact
     :param extension_fraction: if truncation correction is applied -> which fraction of original image width is appended
     to both sides of the image
     :param datatype: numpy datatype to use for storing the projections; should be one of float16, float32, float64
+    :param do_beam_hardening_correction: whether or not polynomial beam hardening correction is applied
+    :param poly: coefficients for beam hardening correction
     :return:
     '''
     metadata = read_metadata(str(projections_file))
@@ -166,7 +177,6 @@ def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_fact
         projection_height = int(np.ceil(np.sum(keep_rows) / downsample_factor))
     else:
         projection_height = int(np.ceil(float(metadata['image_height']) / downsample_factor))
-    print('Preprocessing projections')
     if do_truncation_correction:
         projections = np.zeros((int(np.ceil(num_images / angular_factor)),
                                 projection_height,
@@ -198,13 +208,16 @@ def read_and_preprocess_txrm(projections_file, downsample_factor=1, angular_fact
             projection = downsample(projection, downsample_factor, angular_factor)
         if do_truncation_correction:
             projection = truncation_correction(projection, extension_fraction=extension_fraction)
+        if do_beam_hardening_correction:
+            projection = apply_polynomial(projection, poly)
         projections[i, :, :] = np.squeeze(projection)
 
     return projections
 
 
 def read_single_projection(projections_file, downsample_factor=1, angular_factor=1, do_truncation_correction=True,
-                             extension_fraction=0.1, dose_reduction=1., datatype=np.float32, keep_rows=None):
+                             extension_fraction=0.1, dose_reduction=1., datatype=np.float32, keep_rows=None,
+                             do_beam_hardening_correction=False, poly=None):
     metadata = read_metadata(str(projections_file))
     reference = metadata['reference']
     if not dose_reduction == 1.:
@@ -223,6 +236,8 @@ def read_single_projection(projections_file, downsample_factor=1, angular_factor
             projection = downsample(projection, downsample_factor, angular_factor)
         if do_truncation_correction:
             projection = truncation_correction(projection, extension_fraction=extension_fraction)
+        if do_beam_hardening_correction:
+            projection = apply_polynomial(projection, poly)
         break
 
     return projection
