@@ -423,6 +423,9 @@ def read_ole_metadata(ole):
         'pixel_size': _read_ole_value(ole, 'ImageInfo/pixelsize', '<f'),
         'cam_pixel_size': _read_ole_value(ole, 'ImageInfo/CamPixelSize', '<f'),
         #'reference_filename': _read_ole_value(ole, 'ImageInfo/referencefile', '<260s'),
+        'reference_exposure_time': _read_ole_value(ole, 'ReferenceData/ExpTime', '<f'),
+        'reference_current': _read_ole_value(ole, 'ReferenceData/XrayCurrent', '<f'),
+        'reference_voltage': _read_ole_value(ole, 'ReferenceData/XrayVoltage', '<f'),
         'reference_data_type': _read_ole_value(ole, 'referencedata/DataType', '<1I'),
         'image_data_type': _read_ole_value(ole, 'ImageInfo/DataType', '<1I'),
         # NOTE: converting theta to radians from degrees
@@ -445,7 +448,7 @@ def read_ole_metadata(ole):
         'power': _read_ole_value(ole, 'AcquisitionSettings/SrcPower', '<f'),
         'exposure_time': _read_ole_value(ole, 'AcquisitionSettings/ExpTime', '<f'),
         'binning': _read_ole_value(ole, 'AcquisitionSettings/Binning', '<I'),
-        'filter': _read_ole_value(ole, 'AcquisitionSettings/SourceFilterName', '<260s').decode('utf-8').rstrip('\x00')
+        'filter': _read_ole_value(ole, 'AcquisitionSettings/SourceFilterName', '<260s')#.decode('utf-8').rstrip('\x00')
         #'temperatures': _read_ole_arr(ole, 'TemperatureInfo/Temperatures', "<{0}f".format(4 * number_of_images)),
         #'amc_drift_x_shifts': _read_ole_arr(ole, 'Alignment/AMCDriftXShifts', "<{0}f".format(number_of_images)),
         #'amc_drift_y_shifts': _read_ole_arr(ole, 'Alignment/AMCDriftYShifts', "<{0}f".format(number_of_images)),
@@ -473,6 +476,11 @@ def read_ole_metadata(ole):
         #'absorption_scale_factor': _read_ole_value(ole, 'ImageInfo/AbsorptionScaleFactor', '<f'),
         #'absorption_scale_offset': _read_ole_value(ole, 'ImageInfo/AbsorptionScaleOffset', '<f')
     }
+
+    try:
+        metadata['filter'] = metadata['filter'].decode('utf-8').rstrip('\x00')
+    except:
+        print('Could not decode filter information.')
     # special case to remove trailing null characters
     reference_filename = _read_ole_value(ole, 'ImageInfo/referencefile', '<260s')
     if reference_filename is not None:
